<?php

namespace Lean\Console\Form\Storage;

use Picea\Ui\Method\{ FormInterface, FormContext, FormContextInterface };

use \Lean\Console\Lib;

use Picea\Compiler\Context;
use Psr\Http\Message\ServerRequestInterface;
use Ulmus\Common\EntityResolver;
use \Ulmus\Entity\InformationSchema\Table;

class DatabaseMigration implements FormInterface
{
    protected ? Lib\DatabaseMigrations $migration;

    protected array $alter = [];

    public function __construct(Lib\DatabaseMigrations $migration)
    {
        $this->migration = $migration;
    }

    public function initialize(FormContextInterface $context) : void
    {
        $context->tableExist = [];

        foreach($this->migration->entities as $entity => $table) {
            $previous = null;
            $this->alter[$entity] = [];
            $connection = $entity::resolveEntity()->sqlAdapter();
            $adapter = $connection->adapter();

            $tableName = $table->tableName();
            $databaseName = $table->databaseName();

            if ( ! $tableName || ! $databaseName ) {
                continue;
            }

            # Query current table's columns
            $tableEntity = $adapter->schemaTable($connection, $databaseName, $tableName);

            if ( $tableEntity ) {
                foreach($table->fieldList(EntityResolver::KEY_COLUMN_NAME, true) as $field => $definition) {
                    if ( $tableEntity->columns->filtersCollection(fn($e) => $e->name === $field)->count() === 0 ) {
                        $this->alter[$entity][$field] = [
                            'previous' => $previous,
                            'action' => 'add',
                            'definition' => $definition,
                        ];
                    }

                    $previous = [
                        'field' => $field,
                        'definition' => $definition
                    ];
                }

                if ( $this->alter[$entity] ) {
                    $context->status[$entity] = [
                        'msg' => "altering",
                        "query" => $entity::repository()
                            ->alterSqlQuery($this->alter[$entity])
                            ->getSqlQuery(true),
                    ];
                }
                else {
                    # if columns are different
                    $context->status[$entity] = [
                        'msg' => "up-to-date",
                        'query' => "",
                    ];
                }
            }
            else {
                $context->status[$entity] = [
                    'msg' => "unexisting",
                    'query' => $entity::repository()
                        ->createSqlQuery()
                        ->getSqlQuery(true),
                ];
            }
        }

        if ( $context->formSent() ) {

        }
    }

    public function validate(FormContextInterface $context) : bool
    {
        return $context->valid();
    }

    public function execute(FormContextInterface $context) : void
    {
        $create = $context->createAll ? explode(',', $context->createAll) : [ $context->create ];

        foreach(array_filter($create) as $entity) {
            $entity::repository()->createTable();
        }

        $alter = $context->alterAll ? explode(',', $context->alterAll) : [ $context->alter ];

        foreach(array_filter($alter) as $entity) {
            $entity::repository()->alterTable($this->alter[$entity]);
        }

        $this->initialize($context);
    }

    public static function getContext(ServerRequestInterface $request, ? string $formName = null) : FormContextInterface
    {
        return new class($request, $formName) extends FormContext {

            public array $status = [];

            public array $actions = [];
        };
    }
}
